#include <Wire.h>

#include "config.h"

// http://www.willpirkle.com/forum/synth-book-chapter-by-chapter/supersaw/
// http://www.8bit-era.cz/arduino-timer-interrupts-calculator.html
// https://github.com/ElectroSmash/pedalshield-uno/blob/master/pedalshield_uno_pedal_examples.zip

#define FREQ_TIMER 16000

#define PWM_FREQ 0x00FF	// pwm frequency - 31.3KHz
#define PWM_MODE 1	// Fast (1) or Phase Correct (0)
#define PWM_QTY 2	// 2 PWMs in parallel

volatile uint16_t f   = 440;
volatile uint16_t vol = 32767;
volatile float    mul = 1;

void requestEvent(const int n_bytes)
{
	uint8_t f_high = Wire.read();
	uint8_t f_low  = Wire.read();

	f = (f_high << 8) | f_low;

	uint8_t vol_high = Wire.read();
	uint8_t vol_low  = Wire.read();

	vol = (vol_high << 8) | vol_low;

	digitalWrite(13, !!vol);

	mul = f / double(FREQ_TIMER);
}

void set_pwm(const int16_t v)
{
	OCR1AL = v >> 8;	// high byte
	OCR1BL = v;		// low byte
}

uint16_t frame = 0;

ISR(TIMER0_COMPA_vect)
{
	float pos = fmod(mul * frame, 1.0);

	set_pwm(pos * vol);

	frame = (frame + 1) % FREQ_TIMER;
}

void setup() {
	pinMode(13, OUTPUT);
	digitalWrite(13, HIGH);

	Serial.begin(115200);

	Serial.println(F("1. init"));
	Serial.flush();

	pinMode( 9, OUTPUT);
	pinMode(10, OUTPUT);

	noInterrupts();

	// setup PWM
	TCCR1A = (((PWM_QTY - 1) << 5) | 0x80 | (PWM_MODE << 1)); //
	TCCR1B = ((PWM_MODE << 3) | 0x11); // ck/1
//	TIMSK1 = 0x20; // interrupt on capture interrupt
	ICR1H  = (PWM_FREQ >> 8);
	ICR1L  = (PWM_FREQ & 0xff);
	DDRB  |= ((PWM_QTY << 1) | 0x02); // turn on outputs

	TCCR0A = 0; // set entire TCCR0A register to 0
	TCCR0B = 0; // same for TCCR0B
	TCNT0  = 0; // initialize counter value to 0
	// set compare match register for 16000 Hz increments
	OCR0A = 124; // = 16000000 / (8 * 16000) - 1 (must be <256)
	// turn on CTC mode
	TCCR0B |= (1 << WGM01);
	// Set CS02, CS01 and CS00 bits for 8 prescaler
	TCCR0B |= (0 << CS02) | (1 << CS01) | (0 << CS00);
	// enable timer compare interrupt
	TIMSK0 |= (1 << OCIE0A);

	interrupts();

	Serial.println(F("2. init i2c"));
	Serial.print(F("address: "));
	Serial.println(NODE_ADDRESS);
	Serial.flush();

	Wire.begin(NODE_ADDRESS);	// Activate I2C network
	Wire.onReceive(requestEvent);	// Request attention of master node

	Serial.println(F("3. Go!"));
	Serial.flush();

	digitalWrite(13, LOW);
}

void loop() {
}
